import java.io.*;
import java.net.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

public class PhonebookClient {
    private final String base_url;

    // costruttore
    public PhonebookClient(String base_url) {
        this.base_url = base_url;
    }

    // aggiunta di una voce alla rubrica
    // restituisce VERO in caso di successo, FALSO altrimenti
    public boolean addEntry(String name, String number) {
        String doc = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
        
        // costruzione body in formato XML
        doc += "<entry>\r\n";
        doc += "<name>";
        doc += name;
        doc += "</name>\r\n";
        doc += "<number>";
        doc += number;
        doc += "</number>\r\n";
        doc += "</entry>\r\n";
        int n = doc.length();
        try {
            // costruzione dello URL di interrogazione del web-service
            URL server = new URL(base_url + URLEncoder.encode(name, "UTF-8"));
            HttpURLConnection service = (HttpURLConnection)
            server.openConnection();
            // impostazione header richiesta
            service.setRequestProperty("Content-type", "application/xml");
            service.setRequestProperty("Content-length", Integer.toString(n));
            // impostazione metodo di richiesta POST
            service.setDoOutput(true);
            service.setRequestMethod("POST");
            // invio dati al web-service
            BufferedWriter output = new BufferedWriter(new OutputStreamWriter(service.getOutputStream(), "UTF-8"));
            output.write(doc);
            output.flush();
            output.close();
            // connessione al web-service
            service.connect();
            // verifica stato risposta
            int status = service.getResponseCode();
            if (status != 200 && status != 201)
                return false;
        }
        catch (IOException e) {
            return false;
        }
        return true;
    }

    // aggiornamento di una voce della rubrica
    // restituisce VERO in caso di successo, FALSO altrimenti
    public boolean setNumber(String name, String number) {
        String doc = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
        
        // costruzione body in formato XML
        doc += "<entry>\r\n";
        doc += "<name>";
        doc += name;
        doc += "</name>\r\n";
        doc += "<number>";
        doc += number;
        doc += "</number>\r\n";
        doc += "</entry>\r\n";
        int n = doc.length();
        try {
            // costruzione dello URL di interrogazione del web-service
            URL server = new URL(base_url + URLEncoder.encode(name, "UTF-8"));
            HttpURLConnection service = (HttpURLConnection)
            server.openConnection();
            // impostazione header richiesta
            service.setRequestProperty("Content-type", "application/xml");
            service.setRequestProperty("Content-length",Integer.toString(n));
            // impostazione metodo di richiesta PUT
            service.setDoOutput(true);
            service.setRequestMethod("PUT");
            // invio dati al web-service
            BufferedWriter output = new BufferedWriter(new
            OutputStreamWriter(service.getOutputStream(), "UTF-8"));
            output.write(doc);
            output.flush();
            output.close();
            // connessione al web-service
            service.connect();
            // verifica stato risposta
            int status = service.getResponseCode();
            if (status != 200 && status != 204)
                return false;
        }
        catch (IOException e) {
            return false;
        }
        return true;
    }

    // interrogazione rubrica
    public String getNumber(String name) {
        String line;
        try {
            // costruzione dello URL di interrogazione del web-service
            URL server = new URL(base_url + URLEncoder.encode(name, "UTF-8"));
            HttpURLConnection service = (HttpURLConnection)
            server.openConnection();
            // impostazione header richiesta
            service.setRequestProperty("Accept-Charset", "UTF-8");
            service.setRequestProperty("Accept", "text/xml");
            // impostazione metodo di richiesta GET
            service.setDoInput(true);
            service.setRequestMethod("GET");
            // connessione al web-service
            service.connect();
            // verifica stato risposta
            int status = service.getResponseCode();
            if (status != 200 && status != 201)
                return "?";
            // apertura stream di ricezione dati da web-service
            BufferedReader input = new BufferedReader(new
            InputStreamReader(service.getInputStream(), "UTF-8"));
            // apertura stream di scrittura su file
            BufferedWriter file = new BufferedWriter(new FileWriter("entry.xml"));
            // ciclo di ricezione da web-service e scrittura su file
            while ((line = input.readLine()) != null) {
                file.write(line);
                file.newLine();
            }
            input.close();
            file.flush();
            file.close();
            // costruzione albero DOM del file XML
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.parse("entry.xml");
            // estrazione elemento radice
            Element root = document.getDocumentElement();
            // ricerca elemento "number"
            NodeList list = root.getElementsByTagName("number");
            if (list != null && list.getLength() > 0)
                return list.item(0).getFirstChild().getNodeValue();
        }
        catch (IOException e) {
            return "?";
        }
        catch (ParserConfigurationException e) {
            return "?";
        }
        catch (SAXException e) {
            return "?";
        }
        return "?";
    }

    // eliminazione di una voce della rubrica
    // restituisce VERO in caso di successo, FALSO altrimenti
    public boolean delEntry(String name) {
        String line;
        
        try {
            // costruzione dello URL di interrogazione del web-service
            URL server = new URL(base_url + URLEncoder.encode(name, "UTF-8"));
            HttpURLConnection service = (HttpURLConnection)server.openConnection();
            // impostazione metodo di richiesta DELETE
            service.setRequestMethod("DELETE");
            // connessione al web-service
            service.connect();
            int status = service.getResponseCode();
            if (status != 200 && status != 204)
                return false;
        }
        catch (IOException e) {
            return false;
        }
        return true;
    }

    // test
    public static void main(String args[]) {
        String number;
        boolean result;
        PhonebookClient phonebook = new PhonebookClient("http://127.0.0.1:8084/Phonebook/");
        
        result = phonebook.addEntry("Pippo", "111111");
        if (result)
            System.out.println("Aggiunto Pippo.");
        else
            System.out.println("Errore aggiunta Pippo!");
        result = phonebook.addEntry("Pluto", "222222");
        if (result)
            System.out.println("Aggiunto Pluto.");
        else
            System.out.println("Errore aggiunta Pluto!");
        number = phonebook.getNumber("Pippo");
        System.out.println("Il numero di Pippo e’ " + number);
        result = phonebook.setNumber("Pippo", "999999");
        if (result)
            System.out.println("Modificato numero Pippo.");
        else
            System.out.println("Errore modifica numero Pippo!");
        number = phonebook.getNumber("Pippo");
        System.out.println("Il numero di Pippo e’ " + number);
        number = phonebook.getNumber("Paperino");
        System.out.println("Il numero di Paperino e’ " + number);
        result = phonebook.addEntry("Paperino", "333333");
        if (result)
            System.out.println("Aggiunto Paperino.");
        else
            System.out.println("Errore aggiunta Paperino!");
        number = phonebook.getNumber("Paperino");
        System.out.println("Il numero di Paperino e’ " + number);
        result = phonebook.delEntry("Paperino");
        if (result)
            System.out.println("Eliminato Paperino.");
        else
            System.out.println("Errore eliminazione Paperino!");
        number = phonebook.getNumber("Paperino");
        System.out.println("Il numero di Paperino e’ " + number);
      }
}
