import java.io.*;
import java.net.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

public class TestMessage {
    private String base_url;

    // costruttore
    public TestMessage(String base_url) {
        this.base_url = base_url;
    }

    public boolean newMessage(String text) {
        try {
            // costruzione dello URL di interrogazione del web-service
            URL server = new URL(base_url + "?text=" + URLEncoder.encode(text, "UTF-8"));
            HttpURLConnection service = (HttpURLConnection) server.openConnection();
            // impostazione metodo di richiesta POST
            service.setRequestMethod("POST");
            // connessione al web-service
            service.connect();
            // verifica stato risposta
            int status = service.getResponseCode();
            if (status != 200 && status != 201)
                return false;
        }
        catch (IOException e) {
            return false;
        }
        return true;
    }

    public Message[] getAllMessages() {
        Message[] messages;
        String line;
        String text;
        long timestamp;
        
        try {
            // costruzione dello URL di interrogazione del web-service
            URL server = new URL(base_url);
            HttpURLConnection service = (HttpURLConnection) server.openConnection();
            // impostazione header richiesta
            service.setRequestProperty("Accept-Charset", "UTF-8");
            service.setRequestProperty("Accept", "text/xml");
            // impostazione metodo di richiesta GET
            service.setDoInput(true);
            service.setRequestMethod("GET");
            // connessione al web-service
            service.connect();
            // verifica stato risposta
            int status = service.getResponseCode();
            if (status != 200)
                return null;
            // apertura stream di ricezione dati da web-service
            BufferedReader input = new BufferedReader(new InputStreamReader(service.getInputStream(), "UTF-8"));
            // apertura stream di scrittura su file
            BufferedWriter file = new BufferedWriter(new FileWriter("messages.xml"));
            // ciclo di ricezione da web-service e scrittura su file
            while ((line = input.readLine()) != null) {
                file.write(line);
                file.newLine();
            }
            input.close();
            file.flush();
            file.close();
            // costruzione albero DOM del file XML
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.parse("messages.xml");
            // estrazione elemento radice
            Element root = document.getDocumentElement();
            // ricerca elemento "message"
            NodeList list = root.getElementsByTagName("message");
            if (list == null || list.getLength() == 0)
                return null;
            messages = new Message[list.getLength()];
            for (int i=0; i<list.getLength(); i++) {    
                Element message = (Element)list.item(i);
                NodeList tmp = message.getElementsByTagName("text");
                if (tmp != null && tmp.getLength() > 0) {
                    text = tmp.item(0).getFirstChild().getNodeValue();
                    tmp = message.getElementsByTagName("timestamp");
                    if (tmp != null && tmp.getLength() > 0) {
                        timestamp = Long.parseLong(tmp.item(0).getFirstChild().getNodeValue());
                        messages[i] = new Message(timestamp, text);
                    }
                }
            }
            return messages;
        }
        catch (IOException e) {
            return null;
        }
        catch (ParserConfigurationException e) {
            return null;
        }
        catch (SAXException e) {
            return null;
        }
    }

    public Message getMessage(long timestamp) {
        Message message;
        String line;
        String text;
                
        try {
            // costruzione dello URL di interrogazione del web-service
            URL server = new URL(base_url + URLEncoder.encode(Long.toString(timestamp), "UTF-8"));
            HttpURLConnection service = (HttpURLConnection) server.openConnection();
            // impostazione header richiesta
            service.setRequestProperty("Accept-Charset", "UTF-8");
            service.setRequestProperty("Accept", "text/xml");
            // impostazione metodo di richiesta GET
            service.setDoInput(true);
            service.setRequestMethod("GET");
            // connessione al web-service
            service.connect();
            // verifica stato risposta
            int status = service.getResponseCode();
            if (status != 200)
                return null;
            // apertura stream di ricezione dati da web-service
            BufferedReader input = new BufferedReader(new InputStreamReader(service.getInputStream(), "UTF-8"));
            // apertura stream di scrittura su file
            BufferedWriter file = new BufferedWriter(new FileWriter("message.xml"));
            // ciclo di ricezione da web-service e scrittura su file
            while ((line = input.readLine()) != null) {
                file.write(line);
                file.newLine();
            }
            input.close();
            file.flush();
            file.close();
            // costruzione albero DOM del file XML
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.parse("message.xml");
            // estrazione elemento radice
            Element root = document.getDocumentElement();
            // ricerca elemento "message"
            NodeList list = root.getElementsByTagName("text");
            if (list == null || list.getLength() == 0)
                return null;
            text = list.item(0).getFirstChild().getNodeValue();
            list = root.getElementsByTagName("timestamp");
            if (list == null || list.getLength() == 0)
                return null;
            timestamp = Long.parseLong(list.item(0).getFirstChild().getNodeValue());
            message = new Message(timestamp, text);
            return message;
        }
        catch (IOException e) {
            return null;
        }
        catch (ParserConfigurationException e) {
            return null;
        }
        catch (SAXException e) {
            return null;
        }
    }
    
    
    public boolean delMessage(long timestamp) {
        try {
            // costruzione dello URL di interrogazione del web-service
            URL server = new URL(base_url + URLEncoder.encode(Long.toString(timestamp), "UTF-8"));
            HttpURLConnection service = (HttpURLConnection)server.openConnection();
            // impostazione metodo di richiesta DELETE
            service.setRequestMethod("DELETE");
            // connessione al web-service
            service.connect();
            int status = service.getResponseCode();
            if (status != 200 && status != 204)
                return false;
        }
        catch (IOException e) {
            return false;
        }
        return true;
    }

    public static void main(String args[]) {
        Message[] messages;
        Message message;
        boolean result;
        
        TestMessage test = new TestMessage("http://localhost:8080/Message/");
        
        result = test.newMessage("Ciao!");
        if (result)
            System.out.println("Aggiunto messaggio");
        else
            System.out.println("Errore aggiunta messaggio!");
        result = test.newMessage("Hello!");
        if (result)
            System.out.println("Aggiunto messaggio");
        else
            System.out.println("Errore aggiunta messaggio!");
        
        messages = test.getAllMessages();
        if (messages == null)
            System.out.println("Errore lettura messaggi!");
        else {
            System.out.println("Letti " + messages.length + " messaggi:");
            for (int i=0; i<messages.length; i++)
                System.out.println(messages[i]);
       }
       
       if (messages.length > 0) {
            message = test.getMessage(messages[0].getTimestamp());
            if (message == null)
                System.out.println("Errore lettura messaggio!");
            else
                System.out.println("Messaggio: " + message);
            if (test.delMessage(messages[0].getTimestamp()))
                System.out.println("Messaggio eliminato.");
            else
                System.out.println("Errore eliminazione messaggio");
            
                message = test.getMessage(messages[0].getTimestamp());
            if (message == null)
                System.out.println("Errore lettura messaggio!");
            else
                System.out.println("Messaggio: " + message);
       }
    }
}
