
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.concurrent.*;


public class Tutor extends HttpServlet {
    private ConcurrentHashMap<String, Veicolo> veicoli;
    
    public void init() {
        veicoli = new ConcurrentHashMap<String, Veicolo>();
    }
    
    // richiesta POST
    protected void doPost (HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String targa;
        String url;
        String[] url_section;
        String portale;
        long codice_portale;
        
        // estrazione targa da URL
        url = request.getRequestURL().toString();
        url_section = url.split("/");
        targa = url_section[url_section.length-1];
        if (targa == null) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        if (targa.isEmpty()) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        // estrazione codice portale da query string
        portale = request.getParameter("portale");
        try {
            codice_portale = Long.parseLong(portale);
        }
        catch (NumberFormatException e) {
            response.sendError(400, "Wrong or missing operands!");
            return;
        }
        // aggiornamento lista veicoli
        if (veicoli.containsKey(targa)) {
            veicoli.get(targa).nuovoPassaggio(codice_portale);
        }
        else {
            Veicolo veicolo = new Veicolo(targa);
            veicolo.nuovoPassaggio(codice_portale);
            veicoli.put(targa, veicolo);
        }
        response.setStatus(200); // OK
    }
    
    // richiesta GET
    protected void doGet (HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String targa;
        String url;
        String[] url_section;
        Passaggio[] passaggi;
        
        // estrazione targa da URL
        url = request.getRequestURL().toString();
        url_section = url.split("/");
        targa = url_section[url_section.length-1];
        if (targa == null) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        if (targa.isEmpty()) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        if (!veicoli.containsKey(targa)) {
           response.sendError(404, "Unknow plate!");
           return;
        }
        passaggi = veicoli.get(targa).getPassaggi();
        // scrittura del body della risposta
        response.setContentType("text/xml;charset=UTF-8");
        PrintWriter out = response.getWriter();
        try {
            out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            out.println("<portali targa=\""+targa+"\">");
            for (int n=0; n<passaggi.length; n++) {
                out.println("<portale>");
                out.println("<id>"+passaggi[n].getPortale()+"</id>");
                out.println("<timestamp>"+passaggi[n].getTimestamp().toXMLFormat()+"</timestamp>");
                out.println("</portale>");
            }
            out.println("</portali>");
        }    
        finally {
            out.close();
        }
        response.setStatus(200); // OK
    }
    
    // richiesta informazioni servlet
    public String getServletInfo() {
        return "Tutor";
    }
}
