
import java.sql.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

public class Flotta extends HttpServlet {
    final private String driver = "com.mysql.jdbc.Driver";
    final private String dbms_url = "jdbc:mysql://127.0.0.1:3306/";
    final private String database = "Flotta";
    final private String user = "root";
    final private String password = "";
    private Connection flotta;
    private boolean connected;

    // attivazione servlet (connessione a DBMS)
    public void init() {
        String url = dbms_url+database;
        try {
            Class.forName(driver);
            flotta = DriverManager.getConnection(url, user, password);
            connected = true;
        }
        catch (SQLException e) {
            connected = false;
        }
        catch (ClassNotFoundException e) {
            connected = false;
        }
    }

    // disattivazione servlet (disconnessione da DBMS)
    public void destroy() {
        try {
            flotta.close();
        }
        catch (SQLException e) {
        }
    }

    // richiesta GET
    protected void doGet (HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String veicolo, data, ora;
        String url;
        String[] url_section;
        String latitudine, longitudine;
        
        // verifica stato connessione a DBMS
        if (!connected) {
            response.sendError(500, "DBMS server error!");
            return;
        }
        // estrazione veicolo da URL
        url = request.getRequestURL().toString();
        url_section = url.split("/");
        veicolo = url_section[url_section.length-1];
        if (veicolo == null) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        if (veicolo.isEmpty()) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        // estrazione data da query string
        data = request.getParameter("data");
        try {
            // ricerca posizioni veicolo nel database
            Statement statement = flotta.createStatement();
            ResultSet result = statement.executeQuery("SELECT data, ora, latitudine, longitudine FROM Posizioni WHERE veicolo = '" + veicolo + "' AND data = '" + data + "';");
            if (!result.next()) {
                response.sendError(404, "Vehicle or positions not found!");
                result.close();
                statement.close();
                return;
            }
            // scrittura del body della risposta
            response.setContentType("text/xml;charset=UTF-8");
            PrintWriter out = response.getWriter();
            try {
                out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
                out.println("<posizioni veicolo=\""+veicolo+"\">");
                do {
                    latitudine = result.getString(3);
                    longitudine = result.getString(4);
                    data = result.getString(1);
                    ora = result.getString(2);
                    out.println("<posizione>");
                    out.println("<latitudine>"+latitudine+"</latitudine>");
                    out.println("<longitudine>"+longitudine+"</longitudine>");
                    out.println("<data>"+data+"</data>");
                    out.println("<ora>"+ora+"</ora>");
                    out.println("</posizione>");
                } while (result.next());
                out.println("</posizioni>");
            }
            catch (SQLException e) {
                response.sendError(500, "DBMS server error!");
            }
            finally {
                out.close();
                result.close();
                statement.close();
            }
            response.setStatus(200); // OK
        }
        catch (SQLException e) {
            response.sendError(500, "DBMS server error!");
        }
    }

    // richiesta POST
    protected void doPost (HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String veicolo;
        String url;
        String[] url_section;
        String _latitudine, _longitudine;
        float latitudine, longitudine;

        // verifica stato connessione a DBMS
        if (!connected) {
            response.sendError(500, "DBMS server error!");
            return;
        }
        // estrazione targa da URL
        url = request.getRequestURL().toString();
        url_section = url.split("/");
        veicolo = url_section[url_section.length-1];
        if (veicolo == null) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        if (veicolo.isEmpty()) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        // estrazione codice portale da query string
        _latitudine = request.getParameter("latitudine");
        _longitudine = request.getParameter("longitudine");
        try {
            latitudine = Float.parseFloat(_latitudine);
            longitudine = Float.parseFloat(_longitudine);
        }
        catch (NumberFormatException e) {
            response.sendError(400, "Wrong or missing operands!");
            return;
        }
        try {
             // aggiunta posizione nel database
             Statement statement = flotta.createStatement();
             if (statement.executeUpdate("INSERT Posizioni(veicolo, latitudine, longitudine, data, ora) VALUES('" + veicolo + "', " + latitudine + ", " + longitudine + ", DATE(NOW()), TIME(NOW()));") <= 0) {
               response.sendError(403, "Name exist!");
               statement.close();
               return;
            }
            statement.close();
        }
        catch (SQLException e) {
             response.sendError(500, "DBMS server error!");
             return;
        }
        response.setStatus(201); // OK
    }

    // richiesta informazioni servlet
    public String getServletInfo() {
        return "FlottaDB";
    }
}
