package it.zanichelli.jerseyphonebook;

import javax.ws.rs.core.*;
import javax.ws.rs.*;
import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

@ApplicationPath("/")
@Path("")
public class JerseyPhonebook extends Application {
    private static Phonebook phonebook;

    // attivazione web-service (caricamento rubrica da file)
    public JerseyPhonebook() {
        super();
        if (phonebook == null) {
            phonebook = new Phonebook();
            try {
                phonebook.loadBook();
            }
            catch (IOException e) {
            }
        }
    }

    @GET
    @Produces("text/xml")
    @Path("/{name}")
    public String getNumber(@PathParam(value="name") String name) {
        String number;
        String answer;

        try {
            // ricerca nominativo nella rubrica
            synchronized(phonebook) {
                number = phonebook.getNumber(name);
            }
            // generazione del body della risposta
            answer = "<entry>\r\n";
            answer += "<name>";
            answer += name;
            answer += "</name>\r\n";
            answer += "<number>";
            answer += number;
            answer += "</number>\r\n";
            answer += "</entry>\r\n";
        }
        catch (PhonebookException e) {
            throw new WebApplicationException(404); // not found
        }
        return answer;
    }

    @POST
    @Path("/{name}")
    @Consumes("application/xml")
    public Response addEntry(String content, @PathParam(value="name") String url_name) {
        BufferedWriter writer;

        try {
            // scrittura nel file "entry.xml" del body della richiesta
            writer = new BufferedWriter(new FileWriter("entry.xml"));
            writer.write(content);
            writer.flush();
            writer.close();
            // estrazione dei valori degli elementi "name" e "number" dal file "entry.xml"
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.parse("entry.xml");
            Element root = document.getDocumentElement();
            NodeList list = root.getElementsByTagName("name");
            String name = null;
            if (list != null && list.getLength() > 0)
                name = list.item(0).getFirstChild().getNodeValue();
            list = root.getElementsByTagName("number");
            String number = null;
            if (list != null && list.getLength() > 0)
                number = list.item(0).getFirstChild().getNodeValue();
            if (name == null || number == null)
                throw new WebApplicationException(400); // bad request
            if (name.isEmpty() || number.isEmpty())
                throw new WebApplicationException(400); // bad request
            if (!name.equalsIgnoreCase(url_name))
                throw new WebApplicationException(400); // bad request
            try {
                // inserimento voce nella rubrica
                Entry entry = new Entry(name, number);
                synchronized(phonebook) {
                    phonebook.addEntry(entry);
                    phonebook.saveBook();
                }
            }
            catch (PhonebookException e) {
                throw new WebApplicationException(403); // forbidden
            }
        }
        catch (ParserConfigurationException e) {
            throw new WebApplicationException(500); // internal server error
        }
        catch (SAXException e) {
            throw new WebApplicationException(500); // internal server error
        }
        catch (IOException e) {
            throw new WebApplicationException(500); // internal server error
        }
        return Response.status(201).build(); // OK, created
    }

    @PUT
    @Consumes("application/xml")
    @Path("/{name}")
    public Response setNumber(String content, @PathParam(value="name") String url_name) {
        BufferedWriter writer;

        try {
            // scrittura nel file "entry.xml" del body della richiesta
            writer = new BufferedWriter(new FileWriter("entry.xml"));
            writer.write(content);
            writer.flush();
            writer.close();
            // estrazione dei valori degli elementi "name" e "number" dal file "entry.xml"
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.parse("entry.xml");
            Element root = document.getDocumentElement();
            NodeList list = root.getElementsByTagName("name");
            String name = null;
            if (list != null && list.getLength() > 0)
                name = list.item(0).getFirstChild().getNodeValue();
            list = root.getElementsByTagName("number");
            String number = null;
            if (list != null && list.getLength() > 0)
                number = list.item(0).getFirstChild().getNodeValue();
            if (name == null || number == null)
                throw new WebApplicationException(400); // bad request
            if (name.isEmpty() || number.isEmpty())
                throw new WebApplicationException(400); // bad request
            if (!name.equalsIgnoreCase(url_name))
                throw new WebApplicationException(400); // bad request
            try {
                // inserimento voce nella rubrica
                synchronized(phonebook) {
                    phonebook.setNumber(name, number);
                    phonebook.saveBook();
                }
            }
            catch (PhonebookException e) {
                throw new WebApplicationException(400); // bad request
            }
        }
        catch (ParserConfigurationException e) {
            throw new WebApplicationException(500); // internal server error
        }
        catch (SAXException e) {
            throw new WebApplicationException(500); // internal server error
        }
        catch (IOException e) {
            throw new WebApplicationException(500); // internal server error
        }
        return Response.status(204).build(); // OK, no content
    }

    @DELETE
    @Path("/{name}")
    public Response delEntry(String content, @PathParam(value="name") String name) {
        try {
            // eliminazione voce dalla rubrica
            synchronized(phonebook) {
                phonebook.delEntry(name);
                phonebook.saveBook();
            }
        }
        catch (PhonebookException e) {
            throw new WebApplicationException(400); // bad request
        }
        catch (IOException e) {
            throw new WebApplicationException(500); // internal server error
        }
        return Response.status(204).build(); // OK, no content
    }
}
